<?php
/*
 * (c) Logiless Inc.
 */

namespace Plugin\Logiless\Controller;

use Eccube\Application;
use Eccube\Entity\Master\OrderStatus;
use Eccube\Entity\Shipping;
use Eccube\Repository\Master\OrderStatusRepository;
use Eccube\Repository\OrderRepository;
use Eccube\Repository\ShippingRepository;
use Eccube\Service\OrderStateMachine;
use Plugin\Logiless\Dao\OrderDao;
use Plugin\Logiless\E2V\OrderE2V;
use Plugin\Logiless\Exception\LogilessException;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Annotation\Route;

/**
 * Class OrdersController
 * @package Plugin\Logiless\Controller
 */
class OrdersController extends AbstractController
{
    /**
     * @var OrderDao
     */
    protected $orderDao;

    /**
     * @var OrderRepository
     */
    protected $orderRepository;

    /**
     * @var ShippingRepository
     */
    protected $shippingRepository;

    /**
     * @var OrderStatusRepository
     */
    protected $orderStatusRepository;

    /**
     * @var OrderStateMachine
     */
    protected $orderStateMachine;

    /**
     * OrdersController constructor.
     *
     * @param OrderDao $orderDao
     * @param OrderRepository $orderRepository
     * @param ShippingRepository $shippingRepository
     * @param OrderStatusRepository $orderStatusRepository
     * @param OrderStateMachine $orderStateMachine
     */
    public function __construct(
        OrderDao $orderDao,
        OrderRepository $orderRepository,
        ShippingRepository $shippingRepository,
        OrderStatusRepository $orderStatusRepository,
        OrderStateMachine $orderStateMachine
    ) {
        $this->orderDao = $orderDao;
        $this->orderRepository = $orderRepository;
        $this->shippingRepository = $shippingRepository;
        $this->orderStatusRepository = $orderStatusRepository;
        $this->orderStateMachine = $orderStateMachine;
    }

    /**
     * @Route("/plugin/logiless/orders.json", name="plugin_logiless_orders")
     * @param Request $request
     *
     * @return Response
     */
    public function index(Request $request)
    {
        return parent::index($request);
    }

    /**
     * @param Request $request
     *
     * @return JsonResponse
     * @throws LogilessException
     */
    protected function process(Request $request)
    {
        switch ($request->getMethod()) {
            case Request::METHOD_GET:
                return $this->onGet($request);

            case Request::METHOD_PATCH:
                return $this->onPatch($request);

            default:
        }
        throw new NotFoundHttpException();
    }

    /**
     * @param Request $request
     *
     * @return JsonResponse
     */
    protected function onGet(Request $request)
    {
        return $this->createResponse(OrderE2V::convert($this->orderDao->getOrders($request)));
    }

    /**
     * @param Application $app
     * @param Request $request
     *
     * @return JsonResponse
     * @throws LogilessException
     */
    protected function onPatch(Request $request)
    {
        // JSONデータをパース
        $data = json_decode($request->getContent(), true);

        if (is_null($data)) {
            throw LogilessException::onBadRequest(LogilessException::ERROR_INVALID_JSON_FORMAT);
        }

        // 変更後の値を取得
        if (!isset($data['orders']) || !isset($data['status'])) {
            throw LogilessException::onBadRequest(LogilessException::ERROR_INVALID_JSON_STRUCTURE);
        }

        $orderStatus = $this->orderStatusRepository->find($data['status']);

        if (!$orderStatus instanceof OrderStatus) {
            throw LogilessException::onBadRequest(LogilessException::ERROR_STATUS_ID_DOES_NOT_EXIST);
        }

        // Processing
        $errors = array();

        foreach ($data['orders'] as $orderData) {
            if (!isset($orderData['id'])) {
                throw LogilessException::onBadRequest(LogilessException::ERROR_INVALID_JSON_STRUCTURE);
            }

            if (isset($orderData['shipping_id'])) {
                $shippings = $this->shippingRepository->findBy(
                    [
                        'id' => $orderData['shipping_id'],
                        'Order' => $orderData['id'],
                    ]
                );

            } else {
                $shippings = $this->shippingRepository->findBy(
                    [
                        'Order' => $orderData['id'],
                    ]
                );
            }

            foreach ($shippings as $shipping) {
                /* @var $shipping Shipping */

                if ($this->orderStateMachine->can($shipping->getOrder(), $orderStatus)) {
                    if ($orderStatus->getId() == OrderStatus::DELIVERED) {
                        if (!$shipping->isShipped()) {
                            $shipping->setShippingDate(new \DateTime());

                            if (isset($orderData['tracking_number'])) {
                                $shipping->setTrackingNumber($orderData['tracking_number']);
                            }
                        }

                        $allShipped = true;

                        foreach ($shipping->getOrder()->getShippings() as $shippingBrother) {
                            if (!$shippingBrother->isShipped()) {
                                $allShipped = false;
                                break;
                            }
                        }

                        if ($allShipped) {
                            $this->orderStateMachine->apply($shipping->getOrder(), $orderStatus);
                        }

                    } else {
                        $this->orderStateMachine->apply($shipping->getOrder(), $orderStatus);
                    }

                } else {
                    $from = $shipping->getOrder()->getOrderStatus()->getName();
                    $to = $orderStatus->getName();

                    $errors[] = array(
                        'id' => $shipping->getOrder()->getId(),
                        'message' => trans(
                            'admin.order.failed_to_change_status',
                            array(
                                '%name%' => $shipping->getOrder()->getId(),
                                '%from%' => $from,
                                '%to%' => $to,
                            )
                        ),
                    );
                }
            }
        }

        $this->entityManager->flush();

        if (0 < count($errors)) {
            return $this->createResponse(array('errors' => $errors), 400);

        } else {
            return $this->createResponse(array());
        }
    }
}
